#include "crutil.hh"
#include <crdb.h>
#include <stdexcept>

double CRData::chi_squared(const std::valarray<double>& y_aligned) const {
    if (y_aligned.size() != size()) {
        throw std::runtime_error("input array not aligned");
    }
    if (skewed_error) {
        double chi2 = 0;
        for (size_t i = 0; i < size(); ++i) {
            double diff = y_aligned[i] - y(i);
            chi2 += std::pow(diff / (diff < 0 ? ey1(i) : ey2(i)), 2);
        }
        return chi2;
    } else {
        return std::pow((y_aligned - y()) / ey1(), 2).sum();
    }
}

bool CRData::is_from_experiment(const std::string& exp) const {
    // return 0 ==
    //        _CRUTIL::strncasecmp(exp.c_str(), experiment.c_str(), exp.size());
    return _CRUTIL::is_substr_i(experiment, exp);
}

bool CRData::is_quantity(const std::string& quan) const {
    // return 0 ==
    //        _CRUTIL::strncasecmp(quan.c_str(), x_quantity.c_str(), (size_t)-1);
    return x_quantity == quan;
}

double CRData::x_min(void) const {
    if (has_bin) {
        return at(0).x_lower;
    } else {
        if (size() == 1) {
            return x(0);
        } else {
            // todo: support linear
            return x(0) * x(0) / x(1);
        }
    }
}

double CRData::x_max(void) const {
    const size_t last = size() - 1;
    if (has_bin) {
        return at(last).x_upper;
    } else {
        if (last == 0) {
            return x(last);
        } else {
            // todo: support linear
            return x(last) * x(last) / x(last - 1);
        }
    }
}

void CRData::list(const std::vector<std::shared_ptr<CRData>>& lst,
                  std::ostream& os) {
    size_t len_name(0), len_exp(0);
    for (const auto& pd : lst) {
        if (len_name < pd->y_quantity.size()) {
            len_name = pd->y_quantity.size();
        }
        if (len_exp < pd->experiment.size()) {
            len_exp = pd->experiment.size();
        }
    }
    for (const auto& pd : lst) {
        char buf[1024];
        std::sprintf(buf, //
                     ("%-" + std::to_string(len_name + 3) +
                      "s   %-5s    %.1e ~ %.1e   %2ld    %-" +
                      std::to_string(len_exp + 3) + "s  %s")
                         .c_str(),
                     pd->y_quantity.c_str(), pd->x_quantity.c_str(),
                     pd->x().min(), pd->x().max(), pd->size(),
                     pd->experiment.c_str(), pd->cite.c_str());
        os << std::string(buf) << std::endl;
    }
}
